package italo.mlp_xor;

import italo.mlp.MLP;
import italo.mlp.RNA;
import italo.mlp.RNAConfig;
import java.text.DecimalFormat;

public class Main {
    
    public static void main( String[] args ) {                
        // possibilidades de entradas binárias conforme tabela verdade
        double[][] dados = {
            {0, 0}, {0, 1}, {1, 0}, {1, 1}
        };
        
        // saídas desejadas que representam as saídas da operação XOR aplicada a cada entrada
        double[][] saidasEsperadas = {
            {0}, {1}, {1}, {0}
        };
        
        // dois neurônios para a camada de entrada, dois para a intermediária e um para a de saída
        int[] quantNeursPorCamada = {
            2, 2, 1
        };
        
        MLP mlp = new MLP();
                        
        // configuração da rede
        RNAConfig config = new RNAConfig();
        config.setNEpocas( 1000000 ); 
        config.setTaxaAprendizado( 0.5 ); 
        config.setEntradas( dados ); 
        config.setSaidasEsperadas( saidasEsperadas ); 
        config.setQuantNeursPorCamada( quantNeursPorCamada );
        config.setPesoIniRaio( 0.1 ); // durante a criação da rede, são gerados pesos iniciais entre -0.1 e 0.1
        config.setBiasIniRaio( 1 ); // durante a criação da rede, são gerados biases iniciais entre -1 e 1 

        RNA rna = mlp.criaRNA( config );  // criação da rede de acordo com as configurações acima
        mlp.treinar( rna ); // treinamento da rede após a criação
                
        // após a rede treinada, aqui, é feito o cálculo das saídas submetendo-se as entradas a rede
        for( int i = 0; i < dados.length; i++ ) {                                                
            double[] d = dados[i];            
            double[] saida = mlp.calculaSaida( rna, d );
            
            System.out.print( "("+d[0]+",  "+d[1] +") -->  " );
            for( int j = 0; j < saida.length; j++ ) {
                System.out.println( new DecimalFormat( "0.000000" ).format( saida[j] ) );            
            }
        }        
    }
    
}
