package italo.mlp;

import italo.util.ArrayUtil;

public class RNA implements RNADados {
        
    private double[][][] pesos;
    private double[][] biases;
    private double[][] somas;
    private double[][] saidas;
    
    private double[][] deltas;
    
    private final RNAConfig config;
    private final ArrayUtil arrayUtil = new ArrayUtil();
    
    public RNA( RNAConfig config ) {
        this.config = config;
    }

    @Override
    public RNADados copiaDados() {
        RNA rna = new RNA( config );
        rna.setPesos( arrayUtil.copia( pesos ) ); 
        rna.setBiases( arrayUtil.copia( biases ) );
        rna.setSomas( arrayUtil.copia( somas ) );
        rna.setSaidas( arrayUtil.copia( saidas ) );
        rna.setDeltas( arrayUtil.copia( deltas ) );        
        return rna;
    }
    
    public int getNEpocas() {
        return config.getNEpocas();
    }

    public double getPesosTaxaAprendizado() {
        return config.getTaxaAprendizado();
    }
    
    public int[] getNeursPorCamada() {
        return config.getQuantNeursPorCamada();
    }

    public double[][] getEntradas() {
        return config.getEntradas();
    }

    public double[][] getSaidasEsperadas() {
        return config.getSaidasEsperadas();
    }
        
    public double getPesoIniRaio() {
        return config.getPesoIniRaio();
    }
    
    public double getBiasIniRaio() {
        return config.getBiasIniRaio();
    }

    public double[][][] getPesos() {
        return pesos;
    }

    public void setPesos(double[][][] pesos) {
        this.pesos = pesos;
    }

    public double[][] getBiases() {
        return biases;
    }

    public void setBiases(double[][] biases) {
        this.biases = biases;
    }

    public double[][] getSomas() {
        return somas;
    }

    public void setSomas(double[][] somas) {
        this.somas = somas;
    }

    public double[][] getSaidas() {
        return saidas;
    }

    public void setSaidas(double[][] saidas) {
        this.saidas = saidas;
    }        

    public double[][] getDeltas() {
        return deltas;
    }

    public void setDeltas(double[][] deltas) {
        this.deltas = deltas;
    }
    
}
